@@ -1,18 +1,18 @@
 import requests
 
 
-class AccessToken:
+class ApplicationAccessToken:
+    def __init__(self, tenant_id, client_id, client_secret):
+        self.data = {
+            'grant_type': 'client_credentials',
+            'client_id': client_id,
+            'client_secret': client_secret,
+            'tenant_id': tenant_id,
+            'scope': '.default'}
 
-    def __init__(
-            self, code, redirect_uri=None, client_id=None, client_secret=None):
-        self.data = {}
-        self.data['grant_type'] = 'authorization_code'
-        self.data['scope'] = 'user.readwrite'
-        self.data['code'] = code
-        self.data['redirect_uri'] = redirect_uri
-        self.data['client_id'] = client_id
-        self.data['client_secret'] = client_secret
-        self.url = 'https://login.microsoftonline.com/common/oauth2/v2.0/token'
+    def generate_url(self):
+        url = 'https://login.microsoftonline.com/{}/oauth2/v2.0/token'
+        return url.format(self.data['tenant_id'])
 
     def update(self):
         request = requests.post(url=self.url, data=self.data)
@@ -32,16 +32,15 @@ class User:
         self.userid = userid
 
 
-class Team:
-    def __init__(self, code, client_id, client_secret, redirect_uri):
-        self.code = code
-        self.redirect_uri = redirect_uri
+class ApplicationTeam:
+    def __init__(self, tenant_id, client_id, client_secret):
+        self.tenant_id = tenant_id
         self.client_id = client_id
         self.client_secret = client_secret
 
     def update_access_token(self):
-        self.access_token = AccessToken(
-            self.code, self.redirect_uri, self.client_id, self.client_secret)
+        self.access_token = ApplicationAccessToken(
+            self.code, self.tenant_id, self.client_id, self.client_secret)
         self.access_token.update()
 
         if not self.access_token.token:
@@ -82,10 +81,46 @@ class Team:
                 password=password)
             return True
 
-    def update_user(self, username, userid, password=None):
-        self.user = User(username=username, userid=userid)
-        if password is not None:
-            self.user.password = password
+
+class DelegatedAccessToken(ApplicationAccessToken):
+    def __init__(self, code, redirect_uri, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self.data.update({
+            'code': code, 'redirect_uri': redirect_uri,
+            'grant_type': 'authorization_code'})
+
+    def generate_url(self):
+        url = 'https://login.microsoftonline.com/{}/oauth2/v2.0/token'
+        url = url.format(self.data['tenant_id'])
+        return url
+
+
+class DelegatedTeam(ApplicationAccessToken):
+    def generate_get_code_url(self, redirect_uri, state):
+        data = {
+            'redirect_uri': redirect_uri, 'state': state,
+            'response_type': 'code', 'client_id': self.client_id,
+            'scope': '.default', 'state': state}
+
+        url = 'https://login.microsoftonline.com/{}/oauth2/v2.0/authorize?'
+        url = url.format(data['tenant_id'])
+
+        for key in self.data.keys():
+            if not url.endswith('?'):
+                url = url + '&'
+            url = url + '{}={}'.format(key, data[key])
+
+        return url
+
+    def update_access_token(self):
+        self.access_token = DelegatedAccessToken(
+            self.code, self.tenant_id, self.redirect_uri, self.client_id,
+            self.client_secret)
+        self.access_token.update()
+
+        if not self.access_token.token:
+            raise Exception(self.access_token.error)
 
     def create_meeting(self, start_time, end_time, subject):
         self.check()
@@ -112,11 +147,5 @@ class Team:
             self.error = return_values['error']['message']
             return False
         else:
-            print(return_values)
-            # self.meeting_url = (
-            #     'https://teams.microsoft.com/l/meetup-join/'
-            #     '19%3ameeting_ZmRmZGRjNDUtNzRiZS00NzE0LWJlODAtMmE4MzY3OWQ0ZmYw'
-            #     '%40thread.v2/0?context=%7b%22Tid%22%3a%220f306446-c01e-4c96'
-            #     '-9c4c-57d90425f92c%22%2c%22Oid%22%3a%22be01e8dd-44cc-409d-'
-            #     '892a-554a91a04459%22%7d')
+            self.meeting_url = return_values['joinUrl']
             return True
